
!-------------------------------------------------------------------------------
!> module TRACER
!!
!! @par Description
!!          Tracer module
!!
!! @author Team SCALE
!!
!! @par History
!! @li      2013-12-04 (S.Nishizawa)   [new]
!!
!<
!-------------------------------------------------------------------------------
! Warning: This file was generated from atmos-physics/microphysics/scale_tracer.F90.erb.
!          Do not edit this file.
!-------------------------------------------------------------------------------
module scale_tracer
  !-----------------------------------------------------------------------------
  !
  !++ used modules
  !
  use scale_precision
  use scale_stdio
  !-----------------------------------------------------------------------------
  implicit none
  private
  !-----------------------------------------------------------------------------
  !
  !++ Public procedure
  !
  !-----------------------------------------------------------------------------
  !
  !++ Public parameters & variables
  !

#ifdef TRACER
#define EXTM(name) inc_tracer_ ## name ## .h
#define NAME(name) EXTM(name)
#define XSTR(s) # s
#define STR(s) XSTR(s)
  include STR(NAME(TRACER))
#else
  integer, public :: QA

  integer, public :: I_QV
  integer, public :: I_QC
  integer, public :: I_QR
  integer, public :: I_QI
  integer, public :: I_QS
  integer, public :: I_QG
  integer, public :: I_NC
  integer, public :: I_NR
  integer, public :: I_NI
  integer, public :: I_NS
  integer, public :: I_NG

  integer, public :: QQA ! mass tracer (water)
  integer, public :: QQS ! start index for mass tracer
  integer, public :: QQE ! end   index for mass tracer

  integer, public :: QWS ! start index for water tracer
  integer, public :: QWE ! end   index for water tracer
  integer, public :: QIS ! start index for ice tracer
  integer, public :: QIE ! end   index for ice tracer

  character(len=H_SHORT), public, allocatable :: AQ_NAME(:)
  character(len=H_MID)  , public, allocatable :: AQ_DESC(:)
  character(len=H_SHORT), public, allocatable :: AQ_UNIT(:)

  !-----------------------------------------------------------------------------
  !
  !++ tracer index & relationship (MP_dry+AE_dummy+RD_mstrnx)
  !
  !-----------------------------------------------------------------------------

  integer, public :: MP_QA ! number of hydrometeor tracer
  integer, public, allocatable :: I_MP2ALL(:)

  integer, public, allocatable :: I_MP2RD(:)

  integer, public :: AE_QA ! number of aerosol tracer
  integer, public :: I_ae_dummy
  integer, public, allocatable :: I_AE2ALL(:)

  integer, public, allocatable :: I_AE2RD(:)

  character(len=H_SHORT), public :: TRACER_TYPE = 'NONE'
#endif
  !-----------------------------------------------------------------------------
  !
  !++ Private procedure
  !
  public :: TRACER_setup

  !-----------------------------------------------------------------------------
  !
  !++ Private parameters & variables
  !
  !-----------------------------------------------------------------------------
contains
  !-----------------------------------------------------------------------------
  !> Setup
  subroutine TRACER_setup
#ifdef TRACER
#define EXTM2(name) scale_tracer_ ## name
#define _MODNAME(name) EXTM2(name)
#define EXTM3(name) tracer_ ## name ## _setup
#define _SETUP(name) EXTM3(name)
  use _MODNAME(TRACER), only: &
    _SETUP(TRACER)
#else
  use scale_process, only: &
    PRC_MPIstop
  use scale_tracer_dry, only: &
    QA_dry => QA, &
    I_QV_dry => I_QV, &
    I_QC_dry => I_QC, &
    I_QR_dry => I_QR, &
    I_QI_dry => I_QI, &
    I_QS_dry => I_QS, &
    I_QG_dry => I_QG, &
    I_NC_dry => I_NC, &
    I_NR_dry => I_NR, &
    I_NI_dry => I_NI, &
    I_NS_dry => I_NS, &
    I_NG_dry => I_NG, &
    QQA_dry => QQA, &
    QQS_dry => QQS, &
    QQE_dry => QQE, &
    QWS_dry => QWS, &
    QWE_dry => QWE, &
    QIS_dry => QIS, &
    QIE_dry => QIE, &
    MP_QA_dry => MP_QA, &
    AE_QA_dry => AE_QA, &
    I_ae_dummy_dry => I_ae_dummy, &
    AQ_NAME_dry => AQ_NAME, &
    AQ_DESC_dry => AQ_DESC, &
    AQ_UNIT_dry => AQ_UNIT, &
    I_MP2ALL_dry => I_MP2ALL, &
    I_MP2RD_dry => I_MP2RD, &
    I_AE2ALL_dry => I_AE2ALL, &
    I_AE2RD_dry => I_AE2RD, &
    TRACER_dry_setup
  use scale_tracer_kessler, only: &
    QA_kessler => QA, &
    I_QV_kessler => I_QV, &
    I_QC_kessler => I_QC, &
    I_QR_kessler => I_QR, &
    I_QI_kessler => I_QI, &
    I_QS_kessler => I_QS, &
    I_QG_kessler => I_QG, &
    I_NC_kessler => I_NC, &
    I_NR_kessler => I_NR, &
    I_NI_kessler => I_NI, &
    I_NS_kessler => I_NS, &
    I_NG_kessler => I_NG, &
    QQA_kessler => QQA, &
    QQS_kessler => QQS, &
    QQE_kessler => QQE, &
    QWS_kessler => QWS, &
    QWE_kessler => QWE, &
    QIS_kessler => QIS, &
    QIE_kessler => QIE, &
    MP_QA_kessler => MP_QA, &
    AE_QA_kessler => AE_QA, &
    I_ae_dummy_kessler => I_ae_dummy, &
    AQ_NAME_kessler => AQ_NAME, &
    AQ_DESC_kessler => AQ_DESC, &
    AQ_UNIT_kessler => AQ_UNIT, &
    I_MP2ALL_kessler => I_MP2ALL, &
    I_MP2RD_kessler => I_MP2RD, &
    I_AE2ALL_kessler => I_AE2ALL, &
    I_AE2RD_kessler => I_AE2RD, &
    TRACER_kessler_setup
  use scale_tracer_tomita08, only: &
    QA_tomita08 => QA, &
    I_QV_tomita08 => I_QV, &
    I_QC_tomita08 => I_QC, &
    I_QR_tomita08 => I_QR, &
    I_QI_tomita08 => I_QI, &
    I_QS_tomita08 => I_QS, &
    I_QG_tomita08 => I_QG, &
    I_NC_tomita08 => I_NC, &
    I_NR_tomita08 => I_NR, &
    I_NI_tomita08 => I_NI, &
    I_NS_tomita08 => I_NS, &
    I_NG_tomita08 => I_NG, &
    QQA_tomita08 => QQA, &
    QQS_tomita08 => QQS, &
    QQE_tomita08 => QQE, &
    QWS_tomita08 => QWS, &
    QWE_tomita08 => QWE, &
    QIS_tomita08 => QIS, &
    QIE_tomita08 => QIE, &
    MP_QA_tomita08 => MP_QA, &
    AE_QA_tomita08 => AE_QA, &
    I_ae_dummy_tomita08 => I_ae_dummy, &
    AQ_NAME_tomita08 => AQ_NAME, &
    AQ_DESC_tomita08 => AQ_DESC, &
    AQ_UNIT_tomita08 => AQ_UNIT, &
    I_MP2ALL_tomita08 => I_MP2ALL, &
    I_MP2RD_tomita08 => I_MP2RD, &
    I_AE2ALL_tomita08 => I_AE2ALL, &
    I_AE2RD_tomita08 => I_AE2RD, &
    TRACER_tomita08_setup
  use scale_tracer_sn14, only: &
    QA_sn14 => QA, &
    I_QV_sn14 => I_QV, &
    I_QC_sn14 => I_QC, &
    I_QR_sn14 => I_QR, &
    I_QI_sn14 => I_QI, &
    I_QS_sn14 => I_QS, &
    I_QG_sn14 => I_QG, &
    I_NC_sn14 => I_NC, &
    I_NR_sn14 => I_NR, &
    I_NI_sn14 => I_NI, &
    I_NS_sn14 => I_NS, &
    I_NG_sn14 => I_NG, &
    QQA_sn14 => QQA, &
    QQS_sn14 => QQS, &
    QQE_sn14 => QQE, &
    QWS_sn14 => QWS, &
    QWE_sn14 => QWE, &
    QIS_sn14 => QIS, &
    QIE_sn14 => QIE, &
    MP_QA_sn14 => MP_QA, &
    AE_QA_sn14 => AE_QA, &
    I_ae_dummy_sn14 => I_ae_dummy, &
    AQ_NAME_sn14 => AQ_NAME, &
    AQ_DESC_sn14 => AQ_DESC, &
    AQ_UNIT_sn14 => AQ_UNIT, &
    I_MP2ALL_sn14 => I_MP2ALL, &
    I_MP2RD_sn14 => I_MP2RD, &
    I_AE2ALL_sn14 => I_AE2ALL, &
    I_AE2RD_sn14 => I_AE2RD, &
    TRACER_sn14_setup
  use scale_tracer_suzuki10, only: &
    QA_suzuki10 => QA, &
    I_QV_suzuki10 => I_QV, &
    I_QC_suzuki10 => I_QC, &
    I_QR_suzuki10 => I_QR, &
    I_QI_suzuki10 => I_QI, &
    I_QS_suzuki10 => I_QS, &
    I_QG_suzuki10 => I_QG, &
    I_NC_suzuki10 => I_NC, &
    I_NR_suzuki10 => I_NR, &
    I_NI_suzuki10 => I_NI, &
    I_NS_suzuki10 => I_NS, &
    I_NG_suzuki10 => I_NG, &
    QQA_suzuki10 => QQA, &
    QQS_suzuki10 => QQS, &
    QQE_suzuki10 => QQE, &
    QWS_suzuki10 => QWS, &
    QWE_suzuki10 => QWE, &
    QIS_suzuki10 => QIS, &
    QIE_suzuki10 => QIE, &
    MP_QA_suzuki10 => MP_QA, &
    AE_QA_suzuki10 => AE_QA, &
    I_ae_dummy_suzuki10 => I_ae_dummy, &
    AQ_NAME_suzuki10 => AQ_NAME, &
    AQ_DESC_suzuki10 => AQ_DESC, &
    AQ_UNIT_suzuki10 => AQ_UNIT, &
    I_MP2ALL_suzuki10 => I_MP2ALL, &
    I_MP2RD_suzuki10 => I_MP2RD, &
    I_AE2ALL_suzuki10 => I_AE2ALL, &
    I_AE2RD_suzuki10 => I_AE2RD, &
    TRACER_suzuki10_setup
#endif
    implicit none

#ifndef TRACER
    namelist / PARAM_TRACER / &
         TRACER_TYPE
#endif

    integer :: ierr
    !---------------------------------------------------------------------------

    if( IO_L ) write(IO_FID_LOG,*) ''
    if( IO_L ) write(IO_FID_LOG,*) '+++ Module[TRACER]/Categ[COMMON]'

#ifdef TRACER
    call _SETUP(TRACER)
#else
    !--- read namelist
    rewind(IO_FID_CONF)
    read(IO_FID_CONF,nml=PARAM_TRACER,iostat=ierr)

    if( ierr < 0 ) then !--- missing
       if( IO_L ) write(IO_FID_LOG,*) '*** Not found namelist. Default used.'
    elseif( ierr > 0 ) then !--- fatal error
       write(*,*) 'xxx Not appropriate names in namelist PARAM_TRACER. Check!'
       call PRC_MPIstop
    endif
    if( IO_L ) write(IO_FID_LOG,nml=PARAM_TRACER)

    select case (TRACER_TYPE)
    case ("DRY")
       call TRACER_dry_setup
       QA = QA_dry
       I_QV = I_QV_dry
       I_QC = I_QC_dry
       I_QR = I_QR_dry
       I_QI = I_QI_dry
       I_QS = I_QS_dry
       I_QG = I_QG_dry
       I_NC = I_NC_dry
       I_NR = I_NR_dry
       I_NI = I_NI_dry
       I_NS = I_NS_dry
       I_NG = I_NG_dry
       QQA = QQA_dry
       QQS = QQS_dry
       QQE = QQE_dry
       QWS = QWS_dry
       QWE = QWE_dry
       QIS = QIS_dry
       QIE = QIE_dry
       MP_QA = MP_QA_dry
       AE_QA = AE_QA_dry
       I_ae_dummy = I_ae_dummy_dry
       allocate( AQ_NAME(QA) )
       allocate( AQ_DESC(QA) )
       allocate( AQ_UNIT(QA) )
       allocate( I_MP2ALL(MP_QA) )
       allocate( I_MP2RD(MP_QA) )
       allocate( I_AE2ALL(AE_QA) )
       allocate( I_AE2RD(AE_QA) )
       AQ_NAME = AQ_NAME_dry
       AQ_DESC = AQ_DESC_dry
       AQ_UNIT = AQ_UNIT_dry
       I_MP2ALL = I_MP2ALL_dry
       I_MP2RD = I_MP2RD_dry
       I_AE2ALL = I_AE2ALL_dry
       I_AE2RD = I_AE2RD_dry
    case ("KESSLER")
       call TRACER_kessler_setup
       QA = QA_kessler
       I_QV = I_QV_kessler
       I_QC = I_QC_kessler
       I_QR = I_QR_kessler
       I_QI = I_QI_kessler
       I_QS = I_QS_kessler
       I_QG = I_QG_kessler
       I_NC = I_NC_kessler
       I_NR = I_NR_kessler
       I_NI = I_NI_kessler
       I_NS = I_NS_kessler
       I_NG = I_NG_kessler
       QQA = QQA_kessler
       QQS = QQS_kessler
       QQE = QQE_kessler
       QWS = QWS_kessler
       QWE = QWE_kessler
       QIS = QIS_kessler
       QIE = QIE_kessler
       MP_QA = MP_QA_kessler
       AE_QA = AE_QA_kessler
       I_ae_dummy = I_ae_dummy_kessler
       allocate( AQ_NAME(QA) )
       allocate( AQ_DESC(QA) )
       allocate( AQ_UNIT(QA) )
       allocate( I_MP2ALL(MP_QA) )
       allocate( I_MP2RD(MP_QA) )
       allocate( I_AE2ALL(AE_QA) )
       allocate( I_AE2RD(AE_QA) )
       AQ_NAME = AQ_NAME_kessler
       AQ_DESC = AQ_DESC_kessler
       AQ_UNIT = AQ_UNIT_kessler
       I_MP2ALL = I_MP2ALL_kessler
       I_MP2RD = I_MP2RD_kessler
       I_AE2ALL = I_AE2ALL_kessler
       I_AE2RD = I_AE2RD_kessler
    case ("TOMITA08")
       call TRACER_tomita08_setup
       QA = QA_tomita08
       I_QV = I_QV_tomita08
       I_QC = I_QC_tomita08
       I_QR = I_QR_tomita08
       I_QI = I_QI_tomita08
       I_QS = I_QS_tomita08
       I_QG = I_QG_tomita08
       I_NC = I_NC_tomita08
       I_NR = I_NR_tomita08
       I_NI = I_NI_tomita08
       I_NS = I_NS_tomita08
       I_NG = I_NG_tomita08
       QQA = QQA_tomita08
       QQS = QQS_tomita08
       QQE = QQE_tomita08
       QWS = QWS_tomita08
       QWE = QWE_tomita08
       QIS = QIS_tomita08
       QIE = QIE_tomita08
       MP_QA = MP_QA_tomita08
       AE_QA = AE_QA_tomita08
       I_ae_dummy = I_ae_dummy_tomita08
       allocate( AQ_NAME(QA) )
       allocate( AQ_DESC(QA) )
       allocate( AQ_UNIT(QA) )
       allocate( I_MP2ALL(MP_QA) )
       allocate( I_MP2RD(MP_QA) )
       allocate( I_AE2ALL(AE_QA) )
       allocate( I_AE2RD(AE_QA) )
       AQ_NAME = AQ_NAME_tomita08
       AQ_DESC = AQ_DESC_tomita08
       AQ_UNIT = AQ_UNIT_tomita08
       I_MP2ALL = I_MP2ALL_tomita08
       I_MP2RD = I_MP2RD_tomita08
       I_AE2ALL = I_AE2ALL_tomita08
       I_AE2RD = I_AE2RD_tomita08
    case ("SN14")
       call TRACER_sn14_setup
       QA = QA_sn14
       I_QV = I_QV_sn14
       I_QC = I_QC_sn14
       I_QR = I_QR_sn14
       I_QI = I_QI_sn14
       I_QS = I_QS_sn14
       I_QG = I_QG_sn14
       I_NC = I_NC_sn14
       I_NR = I_NR_sn14
       I_NI = I_NI_sn14
       I_NS = I_NS_sn14
       I_NG = I_NG_sn14
       QQA = QQA_sn14
       QQS = QQS_sn14
       QQE = QQE_sn14
       QWS = QWS_sn14
       QWE = QWE_sn14
       QIS = QIS_sn14
       QIE = QIE_sn14
       MP_QA = MP_QA_sn14
       AE_QA = AE_QA_sn14
       I_ae_dummy = I_ae_dummy_sn14
       allocate( AQ_NAME(QA) )
       allocate( AQ_DESC(QA) )
       allocate( AQ_UNIT(QA) )
       allocate( I_MP2ALL(MP_QA) )
       allocate( I_MP2RD(MP_QA) )
       allocate( I_AE2ALL(AE_QA) )
       allocate( I_AE2RD(AE_QA) )
       AQ_NAME = AQ_NAME_sn14
       AQ_DESC = AQ_DESC_sn14
       AQ_UNIT = AQ_UNIT_sn14
       I_MP2ALL = I_MP2ALL_sn14
       I_MP2RD = I_MP2RD_sn14
       I_AE2ALL = I_AE2ALL_sn14
       I_AE2RD = I_AE2RD_sn14
    case ("SUZUKI10")
       call TRACER_suzuki10_setup
       QA = QA_suzuki10
       I_QV = I_QV_suzuki10
       I_QC = I_QC_suzuki10
       I_QR = I_QR_suzuki10
       I_QI = I_QI_suzuki10
       I_QS = I_QS_suzuki10
       I_QG = I_QG_suzuki10
       I_NC = I_NC_suzuki10
       I_NR = I_NR_suzuki10
       I_NI = I_NI_suzuki10
       I_NS = I_NS_suzuki10
       I_NG = I_NG_suzuki10
       QQA = QQA_suzuki10
       QQS = QQS_suzuki10
       QQE = QQE_suzuki10
       QWS = QWS_suzuki10
       QWE = QWE_suzuki10
       QIS = QIS_suzuki10
       QIE = QIE_suzuki10
       MP_QA = MP_QA_suzuki10
       AE_QA = AE_QA_suzuki10
       I_ae_dummy = I_ae_dummy_suzuki10
       allocate( AQ_NAME(QA) )
       allocate( AQ_DESC(QA) )
       allocate( AQ_UNIT(QA) )
       allocate( I_MP2ALL(MP_QA) )
       allocate( I_MP2RD(MP_QA) )
       allocate( I_AE2ALL(AE_QA) )
       allocate( I_AE2RD(AE_QA) )
       AQ_NAME = AQ_NAME_suzuki10
       AQ_DESC = AQ_DESC_suzuki10
       AQ_UNIT = AQ_UNIT_suzuki10
       I_MP2ALL = I_MP2ALL_suzuki10
       I_MP2RD = I_MP2RD_suzuki10
       I_AE2ALL = I_AE2ALL_suzuki10
       I_AE2RD = I_AE2RD_suzuki10
     case default
        write(*,*) 'xxx Unsupported TRACER_TYPE (', trim(TRACER_TYPE), '). Check!'
        call PRC_MPIstop
     end select
#endif

  end subroutine TRACER_setup

end module scale_tracer
